@@ -22,6 +22,8 @@
 /* Handle for the PCM device */
 snd_pcm_t *pcm_handle;
 
+bool can_pause;
+
 #if GBS_BYTE_ORDER == GBS_ORDER_LITTLE_ENDIAN
 #define SND_PCM_FORMAT_S16_NE SND_PCM_FORMAT_S16_LE
 #else
@@ -98,9 +100,18 @@ static long alsa_open(enum plugout_endian *endian, long rate, long *buffer_bytes
 
 	*buffer_bytes = buffer_frames * 4;
 
+	can_pause = snd_pcm_hw_params_can_pause(hwparams);
+
 	return 0;
 }
 
+static void alsa_pause(int pause)
+{
+	int err;
+	if (can_pause && (err = snd_pcm_pause(pcm_handle, pause)))
+		fprintf(stderr, _("snd_pcm_pause failed: %s\n"), snd_strerror(err));
+}
+
 static long is_suspended(snd_pcm_sframes_t retval)
 {
 #ifdef HAVE_ESTRPIPE
@@ -110,18 +121,24 @@ static long is_suspended(snd_pcm_sframes_t retval)
 #endif
 }
 
+static long is_underrun(snd_pcm_sframes_t retval)
+{
+	return retval == -EPIPE;
+}
+
 static ssize_t alsa_write(const void *buf, size_t count)
 {
 	snd_pcm_sframes_t retval;
 
 	do {
 		retval = snd_pcm_writei(pcm_handle, buf, count / 4);
-		if (!is_suspended(retval))
-			break;
-
-		/* resume from suspend */
-		while (snd_pcm_resume(pcm_handle) == -EAGAIN)
-			sleep(1);
+		if (is_suspended(retval)) {
+			/* resume from suspend */
+			while (snd_pcm_resume(pcm_handle) == -EAGAIN)
+				sleep(1);
+		} else if (is_underrun(retval)) {
+			snd_pcm_prepare(pcm_handle);
+		} else break;
 	} while (1);
 	if (retval < 0) {
 		fprintf(stderr, _("snd_pcm_writei failed: %s\n"), snd_strerror(retval));
@@ -140,6 +157,7 @@ const struct output_plugin plugout_alsa = {
 	.name = "alsa",
 	.description = "ALSA sound driver",
 	.open = alsa_open,
+	.pause = alsa_pause,
 	.write = alsa_write,
 	.close = alsa_close,
 };
